<?php

namespace kernel\services;

use DirectoryIterator;
use kernel\helpers\Debug;
use kernel\helpers\Manifest;
use kernel\helpers\RESTClient;
use kernel\models\Option;

class AdminThemeService
{
    protected Option $option;
    protected string $active_theme;
    protected ModuleService $moduleService;


    public function __construct()
    {
        $this->option = new Option();
        $this->findActiveAdminTheme();
        $this->moduleService = new ModuleService();
    }

    public function findActiveAdminTheme(): void
    {
        $model = Option::where("key", "active_admin_theme")->first();
        $this->active_theme = $model->value;
    }

    public function getActiveAdminTheme(): string
    {
        return $this->active_theme;
    }

    public function setActiveAdminTheme(string $theme): void
    {
        $active_admin_theme = Option::where("key", "active_admin_theme")->first();
        $active_admin_theme->value = getConst($theme);
        $active_admin_theme->save();
    }

    public function getActiveAdminThemeInfo(): false|array|string
    {
        return $this->getAdminThemeInfo($this->active_theme);
    }

    public function getAdminThemeInfo(string $theme): false|array|string
    {
        $info = [];
        $theme = getConst($theme);
        $info['path'] = $theme;
        if (file_exists($theme . "/manifest.json")){
            $manifest = file_get_contents($theme . "/manifest.json");
            $manifest = Manifest::getWithVars($manifest);
            $manifest['preview'] = $manifest['resource'] . "/" . $manifest['preview'];
            $info = array_merge($info, $manifest);
        }

        return $info;
    }

    public function isInstall(string $slug): bool
    {
        $adminThemePaths = Option::where("key", "admin_theme_paths")->first();
        $dirs = [];
        if ($adminThemePaths) {
            $path = json_decode($adminThemePaths->value);
            foreach ($path->paths as $p) {
                $dirs[] = getConst($p);
            }
        }
        foreach ($dirs as $dir) {
            foreach (new DirectoryIterator($dir) as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($this->getAdminThemeInfo($fileInfo->getPathname())['slug'] === $slug) {
                    return true;
                }
            }
        }

        return false;
    }

    public function isLastVersion(string $slug): bool
    {
        if ($this->moduleService->isServerAvailable()) {
            $modules_info = RESTClient::request($_ENV['MODULE_SHOP_URL'] . '/api/module_shop/gb_slug');

            $modules_info = json_decode($modules_info->getBody()->getContents(), true);

            $themeInfo = $this->getAdminThemeInfo($slug);
            foreach ($modules_info as $mod) {
                if ($mod['slug'] === $themeInfo['slug'] && $mod['version'] === $themeInfo['version']) {
                    return true;
                }
            }
        }

        return false;
    }
}